package com.blazinggames.retro.shared.search.tftm;

// Java imports
import java.awt.Point;
import java.util.Vector;

public class AIManager 
{
// *********************
// ***** CONSTANTS *****
// *********************

    protected static final int MAX_TANKS = 9;


// *********************
// ***** VARIABLES *****
// *********************

    // configuration variables
    protected Map _map;
    protected int _id;
    protected int _money;
    protected int _turnMoney;
    protected int _factoryMoney;
    protected int _tankCost;

    // factory management
    protected Path _basicPaths[]; // Paths from base to factory
    protected int _factoryStates[]; // -1 = need, 0-8 = tank assigned, 10=owned
    protected Point _factoryPoints[];

    // Tank management
    protected Path _tankPaths[] = new Path[MAX_TANKS];

    // Game message management
    protected Vector _messages = new Vector();


// ************************
// ***** CONSTRUCTORs *****
// ************************
    
    public AIManager(Map map, int id, int money, int turnMoney, int factoryMoney, int tankCost)
    {
        _map = map;
        _id = id;
    	_money = money;
    	_turnMoney = turnMoney;
    	_factoryMoney = factoryMoney;
    	_tankCost = tankCost;
        initialize();
    }


// *****************************************
// ***** METHODS - Initialization aids *****
// *****************************************

    protected void initialize()
    {
        int numFactories = _map._factories.size();
        
        _basicPaths = new Path[numFactories];
        _factoryStates = new int[numFactories];
        _factoryPoints = new Point[numFactories];
        for (int cntr = 0; cntr < numFactories; ++cntr)
        {
            _factoryPoints[cntr] = new Point((Point)_map._factories.elementAt(cntr));
            _basicPaths[cntr] = new Path(_map, _map._bases[_id], _factoryPoints[cntr], 0);
            _factoryStates[cntr] = -1;
        }
    }


// ******************************
// ***** METHODS - Gameflow *****
// ******************************

    protected void updateGameState()
    {
        int cntr, temp;
        
        // remove any factorys that we lost
        for (cntr = 0; cntr < _factoryPoints.length; ++cntr)
        {
            if (_factoryStates[cntr] == 10)
            {
                temp = _map._map.getAttribute(_factoryPoints[cntr].x, _factoryPoints[cntr].y);
                if (temp != (MapTile.TILE_PLAYER1_FACTORY + _id))
                    _factoryStates[cntr] = -1;
            }
            else if (_factoryStates[cntr] >= 0)
            {
                if (_map._tanks[_id][_factoryStates[cntr]].getHealth() <= 0)
                    _factoryStates[cntr] = -1;
            }
        }
    }

    /**
     * @return true if have enough money to build a tank.
    */
    protected boolean addMoney()
    {
        _money += _turnMoney;
        int temp = _factoryMoney * _map._playerFactoryCounts[_id];
        _money += temp;
        
        return (_tankCost < _money);
    }


// ***************************************
// ***** METHODS - Intelligence Aids *****
// ***************************************

    protected void considerNewTank()
    {
        int cntr;

        if (_map.canCreateTank(_id))
        {
            Path target = null;
            int temp = 0;
            for (cntr = 0; cntr < _factoryPoints.length; ++cntr)
            {
                if (_factoryStates[cntr] == -1)
                {
                    if (target == null)
                    {
                        target = _basicPaths[cntr];
                        temp = cntr;
                    }
                    else if (target.getCost() > _basicPaths[cntr].getCost())
                    {
                        target = _basicPaths[cntr];
                        temp = cntr;
                    }
                }
            }
            
            if (target != null)
            {
                _factoryStates[temp] = _map.createTank(_id);
                _money -= _tankCost;
                _tankPaths[_factoryStates[temp]] = new Path(target);
                _messages.addElement("...Created a new tank!");
//System.out.println("Created tank " + _factoryStates[temp]);
            }
        }
    }
    
    protected void moveTanks()
    {
        Point loc, target;
        boolean tankMoving;
        
        for (int cntr = 0; cntr < MAX_TANKS; ++cntr)
        {
            if (_map._tanks[_id][cntr].getHealth() > 0)
            {
                _map._tanks[_id][cntr].startTurn();
                tankMoving = true;
                while (tankMoving)
                {
                    loc = _map._tanks[_id][cntr].getLocation();
                    target = _tankPaths[cntr].getNextNode(loc);
                    if (loc.equals(target))
                        tankMoving = tankGuard(cntr, loc);
                    else
                        tankMoving = tankMove(cntr, loc, target);
                };
            }
        }
    }

    protected boolean tankGuard(int tankNum, Point loc)
    {
        int los, targetDir = -1;
        
        for (int cntr = 0; cntr < 8; ++cntr)
        {
            los = _map.checkLOS(_id, tankNum, cntr);
            if ( (los >= 0) && (los != _id) )
                targetDir = cntr;
        }
        if (targetDir != -1)
        {
            int leftCost, rightCost, cost;
            int curFacing = _map._tanks[_id][tankNum].getFacing();
            leftCost = curFacing - targetDir;
            if (leftCost < 0)
                leftCost = 8 + curFacing - targetDir;
            rightCost = targetDir - curFacing;
            if (rightCost < 0)
                rightCost = 8 - curFacing + targetDir;
            cost = Math.min(leftCost, rightCost) * 10;
            if (cost < _map._tanks[_id][tankNum].getMovement())
            {
                _map._tanks[_id][tankNum].setFacing(targetDir);
                _map._tanks[_id][tankNum].useMovement(cost);
//                _messages.addElement("[DEBUG] Turning from " + curFacing + " to " + targetDir + " costing " + cost);
            }
            else
                return false; // wait until next turn
            return tankShot(tankNum);
        }
        return false;
    }

    protected boolean tankTurn(int tankNum, Point loc, Point target)
    {
        int nface = 0, leftCost, rightCost, cost;
        int curFacing = _map._tanks[_id][tankNum].getFacing();

        if (loc.x < target.x)
        {
            if (loc.y < target.y)
            {
                nface = Tank.FACING_SOUTHEAST;
            }
            else if (loc.y == target.y)
            {
                nface = Tank.FACING_EAST;
            }
            else
            {
                nface = Tank.FACING_NORTHEAST;
            }
        }
        else if (loc.x == target.x)
        {
            if (loc.y < target.y)
            {
                nface = Tank.FACING_SOUTH;
            }
            else
            {
                nface = Tank.FACING_NORTH;
            }

        }
        else // loc.x > target.x
        {
            if (loc.y < target.y)
            {
                nface = Tank.FACING_SOUTHWEST;
            }
            else if (loc.y == target.y)
            {
                nface = Tank.FACING_WEST;
            }
            else
            {
                nface = Tank.FACING_NORTHWEST;
            }
        }
        leftCost = curFacing - nface;
        if (leftCost < 0)
            leftCost = 8 + curFacing - nface;
        rightCost = nface - curFacing;
        if (rightCost < 0)
            rightCost = 8 - curFacing + nface;
        cost = Math.min(leftCost, rightCost) * 10;
        if (cost < _map._tanks[_id][tankNum].getMovement())
        {
            _map._tanks[_id][tankNum].setFacing(nface);
            _map._tanks[_id][tankNum].useMovement(cost);
//            _messages.addElement("[DEBUG] Turning from " + curFacing + " to " + nface + " costing " + cost);
        }
        else
            return false; // wait until next turn
        return true;
    }
    
    protected boolean tankMove(int tankNum, Point loc, Point target)
    {
        if (tankGuard(tankNum, loc) == true)
            return true;
        if (tankTurn(tankNum, loc, target) == false)
            return false;

        int moveLeft = _map._tanks[_id][tankNum].getMovement();
        int losTarget = _map.checkLOS(_id, tankNum, _map._tanks[_id][tankNum].getFacing() );
        if ( (losTarget >= 0) && (losTarget != _id) )
        {
            if (_map._tanks[_id][tankNum].getFireCost() <= moveLeft)
            {
                return tankShot(tankNum);
            }
            else
                return false;
        }
        int forwardCost = _map.costOfMoveForward(_id, tankNum);
        if (forwardCost <= moveLeft)
        {
            int moveResult = _map.moveTankForward(_id, tankNum);
            _messages.addElement("...Tank " + (tankNum + 1) + " Moving from " + loc.x + "," + loc.y + " to " + target.x + "," + target.y);
            if (moveResult > 0)
                _messages.addElement("...Captured a factory!!!");
        }
        else
            return false;
        return true;
    }


    protected boolean tankShot(int tankNum)
    {
        int result = _map.fire(_id, tankNum);
        if (result < 1)
            return false;
        else if (result == 1)
        {
            _messages.addElement("...Tank " + (tankNum + 1) + " Shot at enemy tank!");
        }
        else
        {
            _messages.addElement("...Tank " + (tankNum + 1) + " Destroyed enemy tank!");
        }
        return true;
    }

    
// ****************************************
// ***** METHODS - Start the insanity *****
// ****************************************

    public String[] playDay()
    {
        int cntr, sz;
        _messages.removeAllElements();
        
        updateGameState();
        
        if (addMoney())
        {
            considerNewTank();
        }
        moveTanks();
        
        _messages.addElement("Done");
        sz = _messages.size();
//                System.out.println("Size is " + sz);
        String rs[] = new String[sz];
        for (cntr = 0; cntr < sz; ++cntr)
        {
            try
            {
                rs[cntr] = (String)_messages.elementAt(cntr);
            }
            catch (ArrayIndexOutOfBoundsException impossible)
            {
                if (cntr >= sz)
                    System.out.println("counter is at " + cntr);
                else
                    rs[cntr] = "impossible exception occured";
            }
        }
        return rs;
    }
}
