package com.blazinggames.retro.shared.search.tftm;

// Java imports
import java.awt.*;

// Spelchan imports
import com.blazinggames.urcl.ui.playfield.*;

/**
* Tanks for the Memory Map Tile. This is a special tile that holds a lot more
 * information about the attribute than just what image to show.
 */
public class MapTile
extends TileRenderer
{
    // *********************
    // ***** CONSTANTS *****
    // *********************

    public static final int TILE_ROAD = 0;
    public static final int TILE_DIRT = 1;
    public static final int TILE_GRASS = 2;
    public static final int TILE_FORREST = 3;
    public static final int TILE_HILLS = 4;
    public static final int TILE_WATER = 5;
    public static final int TILE_MOUNTAINS = 6;
    public static final int TILE_NEUTRAL_FACTORY = 7;

    public static final int TILE_PLAYER1_FACTORY = 8;
    public static final int TILE_PLAYER2_FACTORY = 9;
    public static final int TILE_PLAYER3_FACTORY = 10;
    public static final int TILE_PLAYER4_FACTORY = 11;

    public static final int TILE_PLAYER1_BASE = 12;
    public static final int TILE_PLAYER2_BASE = 13;
    public static final int TILE_PLAYER3_BASE = 14;
    public static final int TILE_PLAYER4_BASE = 15;

    public static final int TOTAL_TILES = 16;

    private static final String DEFAULT_DESCS[] =
    {
        "Road", "Dirt", "Grass", "Forest",
        "Hills", "Water", "Mountain", "Factory",
        "1s Factory", "2s Factory", "3s Factory", "4s Factory",
        "1s Base", "2s Base", "3s Base", "4s Base"
    };
    private static final char DEFAULT_SYMBOLS[] = {
        '.', '-', ',', '%', '^', '~', '*', 'f',
        'a', 'b', 'c', 'd', 'A', 'B', 'C', 'D'  };
    private static final int DEFAULT_RATES[] = {
        8, 10, 12, 20, 25, 0, 0, 8, 8, 8, 8, 8, 8, 8, 8, 8 };
    private static final int DEFAULT_BULLET_COSTS[] = {
        1, 1, 1, 2, 3, 1, 5, 2, 2, 2, 2, 2, 2, 2, 2, 2};

    // *********************
    // ***** VARIABLES *****
    // *********************

    protected Image _tilesImage;

    protected String _descriptions[];
    protected char _symbols[];
    protected int _travelSpeeds[];
    protected int _bulletCosts[];


    // ************************
    // ***** CONSTRUCTORs *****
    // ************************

    public MapTile()
    {
        super(20, 20);
        buildDefaultProperties();
    }

    private void buildDefaultProperties()
    {
        _descriptions = new String[TOTAL_TILES];
        _symbols = new char[TOTAL_TILES];
        _travelSpeeds = new int[TOTAL_TILES];
        _bulletCosts = new int[TOTAL_TILES];

        for (int cntr = 0; cntr < TOTAL_TILES; ++cntr)
        {
            _descriptions[cntr] = DEFAULT_DESCS[cntr];
            _symbols[cntr] = DEFAULT_SYMBOLS[cntr];
            _travelSpeeds[cntr] = DEFAULT_RATES[cntr];
            _bulletCosts[cntr] = DEFAULT_BULLET_COSTS[cntr];
        }
    }


    // ***************************************
    // ***** METHODS - From TileRenderer *****
    // ***************************************

    public void render(Graphics g, Component c, int x, int y)
    {
        render(g, c, x, y, 0);
    }

    public void render(Graphics g, Component c, int x, int y, int state)
    {
        if (_tilesImage == null)
        {
            g.setColor(_backfill);
            g.fillRect(x, y, _width, _height);
        }
        else
        {
            g.drawImage(_tilesImage, x, y, x+_width, y+_height, state * _width, 0, state*_width+_width, _height,c);
        }
    }


    // *****************************************************
    // ***** METHODS - Information getting and setting *****
    // *****************************************************

    public void setImage(Image i)
    {
        _tilesImage = i;
    }

    public void setTileProperties(int id, String name, char symbol, int speed, int bullet)
    {
        if (id < TOTAL_TILES)
        {
            _descriptions[id] = name;
            _symbols[id] = symbol;
            _travelSpeeds[id] = speed;
            _bulletCosts[id] = bullet;
        }
    }

    public int parseSymbol(char ch)
    {
        int result = 0;

        for (int cntr = 0; cntr < TOTAL_TILES; ++cntr)
        {
            if (ch == _symbols[cntr])
            {
                result = cntr;
                break;
            }
        }

        return result;
    }

    public String getName(int id)
    {
        return _descriptions[(id % Integer.MAX_VALUE) % TOTAL_TILES];
    }

    public char getSymbol(int id)
    {
        return _symbols[(id % Integer.MAX_VALUE) % TOTAL_TILES];
    }

    public int getTravelSpeed(int id)
    {
        return _travelSpeeds[(id % Integer.MAX_VALUE) % TOTAL_TILES];
    }

    public int getBulletCost(int id)
    {
        return _bulletCosts[(id % Integer.MAX_VALUE) % TOTAL_TILES];
    }
}
